@@ -93,7 +93,6 @@ def add_context(
 
     def middleware(request: fastapi.Request, call_next):
         request.state.engine = engine
-        request.state.s3 = boto3.client("s3")
         request.state.oauth = oauth
         request.state.frontend_port = frontend_port
         request.state.single_project = single_project
@@ -114,9 +113,11 @@ def get_single_project(request: Request) -> typing.Optional[int]:
     return request.state.single_project
 
 
-def get_s3(request: Request):
+def get_s3():
     """Provide an s3 client"""
-    return request.state.s3
+    if not hasattr(tls, "s3"):
+        tls.s3 = boto3.client("s3")
+    return tls.s3
 
 
 def get_oauth(request: Request) -> OAuthConfig:
@@ -516,7 +517,7 @@ def list_images(
     session.commit()
 
     return [
-        web.Image(id=image.id, filepath=image.filepath, labels=labels, status=status)
+        web.Image(id=image.id, filepath=image.filepath, nLabels=labels, status=status)
         for image, labels, status in entries
     ]
 
@@ -573,6 +574,29 @@ def update_label_group_with_label(
         label_group.text[config.name] = label.text if label is not None else None
 
 
+def get_image(
+    project_id: int,
+    image_id: str,
+    user: web.User = fastapi.Depends(get_current_user),
+    session: sa.orm.Session = fastapi.Depends(get_session),
+) -> web.Image:
+    """Get image with labels."""
+    image = (
+        session.query(orm.Image)
+        .filter((orm.Image.id == image_id) & (orm.Image.project_id == project_id))
+        .first()
+    )
+    if image is None:
+        raise fastapi.HTTPException(404, detail="File not found")
+
+    return web.Image(
+        filepath=image.filepath,
+        labels=get_labels(
+            project_id=project_id, image_id=image_id, user=user, session=session
+        ),
+    )
+
+
 def get_labels(
     project_id: int,
     image_id: str,
@@ -1201,6 +1225,7 @@ def build_app():
     app.post("/api/v1/projects/{project_id}")(update_project)
     app.post("/api/v1/projects/{project_id}/images")(create_images)
     app.get("/api/v1/projects/{project_id}/images")(list_images)
+    app.get("/api/v1/projects/{project_id}/images/{image_id}")(get_image)
     app.get("/api/v1/projects/{project_id}/images/{image_id}/file")(get_file)
     app.get("/api/v1/projects/{project_id}/images/{image_id}/labels")(get_labels)
     app.delete("/api/v1/projects/{project_id}/images/{image_id}/labels")(delete_labels)
